import json
from collections import OrderedDict

import pytest

from great_expectations.data_context.util import file_relative_path
from great_expectations.render.renderer import ProfilingResultsOverviewSectionRenderer
from great_expectations.self_check.util import expectationSuiteValidationResultSchema


@pytest.fixture(scope="module")
def datetime_column_evrs():
    """hand-crafted EVRS for datetime columns"""
    with open(
        file_relative_path(__file__, "../fixtures/datetime_column_evrs.json")
    ) as infile:
        return expectationSuiteValidationResultSchema.load(
            json.load(infile, object_pairs_hook=OrderedDict)
        )


def test_ProfilingResultsOverviewSectionRenderer_render_variable_types(
    datetime_column_evrs,
):
    """Build a type table from a type expectations and assert that we correctly infer column type
    for datetime. Other types would be useful to test for as well."""

    content_blocks = []
    ProfilingResultsOverviewSectionRenderer._render_variable_types(
        datetime_column_evrs, content_blocks
    )
    assert len(content_blocks) == 1
    type_table = content_blocks[0].table
    filtered = [row for row in type_table if row[0] == "datetime"]
    assert filtered == [["datetime", "2"]]



"""
This is a template for creating custom RegexBasedColumnMapExpectations.
For detailed instructions on how to use it, please see:
    https://docs.greatexpectations.io/docs/guides/expectations/creating_custom_expectations/how_to_create_custom_regex_based_column_map_expectations
"""

from typing import Dict, Optional

from great_expectations.core.expectation_configuration import ExpectationConfiguration
from great_expectations.exceptions.exceptions import (
    InvalidExpectationConfigurationError,
)
from great_expectations.expectations.regex_based_column_map_expectation import (
    RegexBasedColumnMapExpectation,
    RegexColumnMapMetricProvider,
)


# This class defines the Expectation itself
# <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py ExpectColumnValuesToMatchSomeRegex class_def">
class ExpectColumnValuesToMatchSomeRegex(RegexBasedColumnMapExpectation):
    # </snippet>
    # <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py docstring">
    """TODO: Add a docstring here"""
    # </snippet>

    # These values will be used to configure the metric created by your expectation
    # <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py definition">
    regex_camel_name = "RegexName"
    regex = "regex pattern"
    # </snippet>
    # <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py plural">
    semantic_type_name_plural = None
    # </snippet>

    # These examples will be shown in the public gallery.
    # They will also be executed as unit tests for your Expectation.
    examples = []

    # Here your regex is used to create a custom metric for this expectation
    map_metric = RegexBasedColumnMapExpectation.register_metric(
        regex_camel_name=regex_camel_name,
        regex_=regex,
    )

    # This object contains metadata for display in the public Gallery
    # <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py library_metadata">
    library_metadata = {
        "tags": [],  # Tags for this Expectation in the Gallery
        "contributors": [  # Github handles for all contributors to this Expectation.
            "@your_name_here",  # Don't forget to add your github handle here!
        ],
    }


# </snippet>
if __name__ == "__main__":
    # <snippet name="tests/integration/docusaurus/expectations/examples/regex_based_column_map_expectation_template.py diagnostics">
    ExpectColumnValuesToMatchSomeRegex().print_diagnostic_checklist()
#     </snippet>



from enum import Enum

from great_expectations.types import (
    ColorPalettes,
    FontFamily,
    PrimaryColors,
    SecondaryColors,
    TintsAndShades,
)

# Size
chart_width: int = 800
chart_height: int = 250

# View
chart_border_opacity: float = 0

# Font
font_family = FontFamily.MONTSERRAT
font = font_family.value

#
# Chart Components
#

# Title
title_align = "center"
title_color = PrimaryColors.ORANGE
title_font_size = 15
title_font_weight = 600
title_dy = -10
subtitle_color = PrimaryColors.COAL_GRAY
subtitle_font = font
subtitle_font_size = 14
subtitle_font_weight = 500

# Both Axes
axis_title_color = PrimaryColors.COAL_GRAY
axis_title_font_size = 14
axis_title_font_weight = 500
axis_title_padding = 10
axis_label_color = PrimaryColors.COAL_GRAY
axis_label_font_size = 12
axis_label_font_weight = 500
axis_label_flush = True
axis_label_overlap_reduction = True

# X-Axis Only
x_axis_title_y = 25
x_axis_label_angle = 0
x_axis_label_flush = True
x_axis_grid = True

# Y-Axis Only
y_axis_title_x = -55

# Legend
legend_title_color = PrimaryColors.COAL_GRAY
legend_title_font_size = 12
legend_title_font_weight = 500

# Scale
scale_continuous_padding = 33
scale_band_padding_outer = 1.0

#
# Color Palettes
#

category_color_scheme = ColorPalettes.CATEGORY_5.value
diverging_color_scheme = ColorPalettes.DIVERGING_7.value
heatmap_color_scheme = ColorPalettes.HEATMAP_6.value
ordinal_color_scheme = ColorPalettes.ORDINAL_5.value

#
# Chart Types
#

# Area
fill_opacity = 0.5
fill_color = TintsAndShades.ROYAL_BLUE_20

# Line Chart
line_color = SecondaryColors.ROYAL_BLUE
line_stroke_width = 2.5
line_opacity = 0.9

# Point
point_size = 50
point_color = SecondaryColors.LEAF_GREEN
point_filled = True
point_opacity = 1.0

# Bar Chart
bar_color = SecondaryColors.ROYAL_BLUE
bar_opacity = 0.7
bar_stroke_color = SecondaryColors.MIDNIGHT_BLUE
bar_stroke_width = 1
bar_stroke_opacity = 1.0


class AltairThemes(Enum):
    """
    Altair theme configuration reference:
        https://altair-viz.github.io/user_guide/configuration.html#top-level-chart-configuration
    """

    DEFAULT_THEME = {
        "view": {
            "width": chart_width,
            "height": chart_height,
            "strokeOpacity": chart_border_opacity,
        },
        "font": font,
        "title": {
            "align": title_align,
            "color": title_color,
            "fontSize": title_font_size,
            "fontWeight": title_font_weight,
            "dy": title_dy,
            "subtitleFont": subtitle_font,
            "subtitleFontSize": subtitle_font_size,
            "subtitleColor": subtitle_color,
            "subtitleFontWeight": subtitle_font_weight,
        },
        "axis": {
            "titleFontSize": axis_title_font_size,
            "titleFontWeight": axis_title_font_weight,
            "titleColor": axis_title_color,
            "titlePadding": axis_title_padding,
            "labelFontSize": axis_label_font_size,
            "labelFontWeight": axis_label_font_weight,
            "labelColor": axis_label_color,
            "labelFlush": axis_label_flush,
            "labelOverlap": axis_label_overlap_reduction,
        },
        "axisY": {
            "titleX": y_axis_title_x,
        },
        "axisX": {
            "titleY": x_axis_title_y,
            "labelAngle": x_axis_label_angle,
            "labelFlush": x_axis_label_flush,
            "grid": x_axis_grid,
        },
        "legend": {
            "titleColor": legend_title_color,
            "titleFontSize": legend_title_font_size,
            "titleFontWeight": legend_title_font_weight,
        },
        "range": {
            "category": category_color_scheme,
            "diverging": diverging_color_scheme,
            "heatmap": heatmap_color_scheme,
            "ordinal": ordinal_color_scheme,
        },
        "scale": {
            "continuousPadding": scale_continuous_padding,
            "bandPaddingOuter": scale_band_padding_outer,
        },
        "area": {
            "color": fill_color,
            "fillOpacity": fill_opacity,
        },
        "line": {
            "color": line_color,
            "strokeWidth": line_stroke_width,
        },
        "point": {
            "size": point_size,
            "color": point_color,
            "filled": point_filled,
            "opacity": point_opacity,
        },
        "bar": {
            "color": bar_color,
            "opacity": bar_opacity,
            "stroke": bar_stroke_color,
            "strokeWidth": bar_stroke_width,
            "strokeOpacity": bar_stroke_opacity,
        },
    }



from .compound_columns_unique import CompoundColumnsUnique
from .multicolumn_sum_equal import MulticolumnSumEqual
from .select_column_values_unique_within_record import (
    SelectColumnValuesUniqueWithinRecord,
)



from typing import Dict, Optional

import pytest

from great_expectations import DataContext
from great_expectations.core.domain import Domain
from great_expectations.core.metric_domain_types import MetricDomainTypes
from great_expectations.rule_based_profiler.helpers.util import (
    get_parameter_value_and_validate_return_type,
)
from great_expectations.rule_based_profiler.parameter_builder import (
    ParameterBuilder,
    ValueCountsSingleBatchParameterBuilder,
)
from great_expectations.rule_based_profiler.parameter_container import (
    ParameterContainer,
    ParameterNode,
)


@pytest.mark.integration
def test_instantiation_value_counts_single_batch_parameter_builder(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    # noinspection PyUnusedLocal
    parameter_builder_0: ParameterBuilder = ValueCountsSingleBatchParameterBuilder(
        name="my_name_0",
        data_context=data_context,
    )


@pytest.mark.integration
def test_value_counts_single_batch_parameter_builder_alice(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = ValueCountsSingleBatchParameterBuilder(
        name="my_name",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "event_type"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    variables: Optional[ParameterContainer] = None
    parameter_builder.build_parameters(
        domain=domain,
        variables=variables,
        parameters=parameters,
        batch_request=batch_request,
        runtime_configuration=None,
    )

    expected_parameter_node_as_dict: dict = {
        "value": {
            "values": [19, 22, 73],
            "weights": [0.3333333333333333, 0.3333333333333333, 0.3333333333333333],
        },
        "details": {
            "metric_configuration": {
                "metric_name": "column.value_counts",
                "domain_kwargs": {"column": "event_type"},
                "metric_value_kwargs": {"sort": "value"},
            },
            "num_batches": 1,
        },
    }

    parameter_node: ParameterNode = get_parameter_value_and_validate_return_type(
        domain=domain,
        parameter_reference=parameter_builder.json_serialized_fully_qualified_parameter_name,
        expected_return_type=None,
        variables=variables,
        parameters=parameters,
    )

    assert parameter_node == expected_parameter_node_as_dict



"""Example Script: How to determine if an Expectation is auto-initializing

This example script is intended for use in online documentation that shows how a user can determine if an Expectation
will work within the auto-initializing framework.

Assert statements are included to ensure that if the behaviour shown in this script breaks it will not pass
tests and will be updated.  These statements can be ignored by users.

Comments with the tags `<snippet>` and `</snippet>` are used to ensure that if this script is updated
the snippets that are specified for use in documentation are maintained.  These comments can be ignored by users.

--documentation--
    https://docs.greatexpectations.io/docs/
"""
# <snippet name="tests/integration/docusaurus/expectations/auto_initializing_expectations/is_expectation_auto_initializing.py is_expectation_self_initializing False">
from great_expectations.expectations.expectation import Expectation

Expectation.is_expectation_self_initializing(name="expect_column_to_exist")
# </snippet>

# <snippet name="tests/integration/docusaurus/expectations/auto_initializing_expectations/is_expectation_auto_initializing.py is_expectation_self_initializing True">
Expectation.is_expectation_self_initializing(name="expect_column_mean_to_be_between")
# </snippet>

# NOTE: The following assertions are only for testing and can be ignored by users.
assert (
    Expectation.is_expectation_self_initializing(name="expect_column_to_exist") is False
)
assert (
    Expectation.is_expectation_self_initializing(
        name="expect_column_mean_to_be_between"
    )
    is True
)



from .my_custom_expectations_store import MyCustomExpectationsStore
from .my_custom_non_core_ge_class import MyCustomNonCoreGeClass
from .my_custom_simple_sqlalchemy_datasource_class import (
    MyCustomSimpleSqlalchemyDatasource,
)
from .my_custom_v2_api_datasource import MyCustomV2ApiDatasource
from .my_custom_v3_api_datasource import MyCustomV3ApiDatasource



from typing import Optional

from great_expectations.core import (
    ExpectationConfiguration,  # noqa: TCH001
    ExpectationValidationResult,  # noqa: TCH001
)
from great_expectations.expectations.expectation import (
    ColumnMapExpectation,
    InvalidExpectationConfigurationError,
    render_evaluation_parameter_string,
)
from great_expectations.render import LegacyRendererType
from great_expectations.render.renderer.renderer import renderer
from great_expectations.render.util import substitute_none_for_missing


class ExpectColumnValuesToMatchLikePatternList(ColumnMapExpectation):
    """Expect the column entries to be strings that match any of a provided list of like pattern expressions.

    expect_column_values_to_match_like_pattern_list is a \
    [Column Map Expectation](https://docs.greatexpectations.io/docs/guides/expectations/creating_custom_expectations/how_to_create_custom_column_map_expectations).

    Args:
        column (str): \
            The column name.
        like_pattern_list (List[str]): \
            The list of like pattern expressions the column entries should match.

    Keyword Args:
        match_on (string): \
            "any" or "all". \
            Use "any" if the value should match at least one like pattern in the list. \
            Use "all" if it should match each like pattern in the list.
        mostly (None or a float between 0 and 1): \
            Successful if at least mostly fraction of values match the expectation. \
            For more detail, see [mostly](https://docs.greatexpectations.io/docs/reference/expectations/standard_arguments/#mostly).

    Other Parameters:
        result_format (str or None): \
            Which output mode to use: BOOLEAN_ONLY, BASIC, COMPLETE, or SUMMARY. \
            For more detail, see [result_format](https://docs.greatexpectations.io/docs/reference/expectations/result_format).
        include_config (boolean): \
            If True, then include the expectation config as part of the result object.
        catch_exceptions (boolean or None): \
            If True, then catch exceptions and include them as part of the result object. \
            For more detail, see [catch_exceptions](https://docs.greatexpectations.io/docs/reference/expectations/standard_arguments/#catch_exceptions).
        meta (dict or None): \
            A JSON-serializable dictionary (nesting allowed) that will be included in the output without \
            modification. For more detail, see [meta](https://docs.greatexpectations.io/docs/reference/expectations/standard_arguments/#meta).

    Returns:
        An [ExpectationSuiteValidationResult](https://docs.greatexpectations.io/docs/terms/validation_result)

        Exact fields vary depending on the values passed to result_format, include_config, catch_exceptions, and meta.

    See Also:
        [expect_column_values_to_match_regex](https://greatexpectations.io/expectations/expect_column_values_to_match_regex)
        [expect_column_values_to_match_regex_list](https://greatexpectations.io/expectations/expect_column_values_to_match_regex_list)
        [expect_column_values_to_not_match_regex](https://greatexpectations.io/expectations/expect_column_values_to_not_match_regex)
        [expect_column_values_to_not_match_regex_list](https://greatexpectations.io/expectations/expect_column_values_to_not_match_regex_list)
        [expect_column_values_to_match_like_pattern](https://greatexpectations.io/expectations/expect_column_values_to_match_like_pattern)
        [expect_column_values_to_not_match_like_pattern](https://greatexpectations.io/expectations/expect_column_values_to_not_match_like_pattern)
        [expect_column_values_to_not_match_like_pattern_list](https://greatexpectations.io/expectations/expect_column_values_to_not_match_like_pattern_list)
    """

    library_metadata = {
        "maturity": "production",
        "tags": ["core expectation", "column map expectation"],
        "contributors": [
            "@great_expectations",
        ],
        "requirements": [],
        "has_full_test_suite": True,
        "manually_reviewed_code": True,
    }

    map_metric = "column_values.match_like_pattern_list"
    success_keys = ("mostly", "like_pattern_list", "match_on")
    default_kwarg_values = {
        "like_pattern_list": None,
        "match_on": "any",
        "row_condition": None,
        "condition_parser": None,  # we expect this to be explicitly set whenever a row_condition is passed
        "mostly": 1,
        "result_format": "BASIC",
        "include_config": True,
        "catch_exceptions": True,
    }
    args_keys = (
        "column",
        "like_pattern_list",
    )

    def validate_configuration(
        self, configuration: Optional[ExpectationConfiguration] = None
    ) -> None:
        """Validates the configuration for the Expectation.

        For `expect_column_values_to_match_like_pattern`
        we require that the `configuraton.kwargs` contain a `like_pattern_list` key that is either a `list` or `dict`.

        Args:
            configuration: The ExpectationConfiguration to be validated.

        Raises:
            InvalidExpectationConfigurationError: The configuraton does not contain the values required by the Expectation
        """
        super().validate_configuration(configuration)
        configuration = configuration or self.configuration
        try:
            assert (
                "like_pattern_list" in configuration.kwargs
            ), "Must provide like_pattern_list"
            assert isinstance(
                configuration.kwargs.get("like_pattern_list"), (list, dict)
            ), "like_pattern_list must be a list"
            assert isinstance(configuration.kwargs.get("like_pattern_list"), dict) or (
                len(configuration.kwargs.get("like_pattern_list")) > 0
            ), "At least one like_pattern must be supplied in the like_pattern_list."
            if isinstance(configuration.kwargs.get("like_pattern_list"), dict):
                assert "$PARAMETER" in configuration.kwargs.get(
                    "like_pattern_list"
                ), 'Evaluation Parameter dict for like_pattern_list kwarg must have "$PARAMETER" key.'

        except AssertionError as e:
            raise InvalidExpectationConfigurationError(str(e))

    @classmethod
    @renderer(renderer_type=LegacyRendererType.PRESCRIPTIVE)
    @render_evaluation_parameter_string
    def _prescriptive_renderer(
        cls,
        configuration: Optional[ExpectationConfiguration] = None,
        result: Optional[ExpectationValidationResult] = None,
        runtime_configuration: Optional[dict] = None,
        **kwargs
    ) -> None:
        runtime_configuration = runtime_configuration or {}
        _ = False if runtime_configuration.get("include_column_name") is False else True
        _ = runtime_configuration.get("styling")
        params = substitute_none_for_missing(  # noqa: F841 # unused
            configuration.kwargs,
            ["column", "mostly", "row_condition", "condition_parser"],
        )



from typing import Dict, Optional
from unittest import mock

import pandas as pd
import pytest

import great_expectations.exceptions as gx_exceptions
from great_expectations import DataContext
from great_expectations.core.domain import Domain
from great_expectations.core.metric_domain_types import MetricDomainTypes
from great_expectations.rule_based_profiler.helpers.util import (
    get_parameter_value_and_validate_return_type,
)
from great_expectations.rule_based_profiler.parameter_builder import (
    HistogramSingleBatchParameterBuilder,
    ParameterBuilder,
)
from great_expectations.rule_based_profiler.parameter_container import (
    ParameterContainer,
    ParameterNode,
)


@pytest.mark.integration
def test_instantiation_histogram_single_batch_parameter_builder(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    # noinspection PyUnusedLocal
    parameter_builder_0: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name_0",
        data_context=data_context,
    )


@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "user_id"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    variables: Optional[ParameterContainer] = None
    parameter_builder.build_parameters(
        domain=domain,
        variables=variables,
        parameters=parameters,
        batch_request=batch_request,
        runtime_configuration=None,
    )

    expected_parameter_node_as_dict: dict = {
        "value": {
            "bins": [397433.0, 4942918.5, 9488404.0],
            "weights": [0.6666666666666666, 0.3333333333333333],
            "tail_weights": [0.0, 0.0],
        },
        "details": {
            "metric_configuration": {
                "metric_name": "column.histogram",
                "domain_kwargs": {"column": "user_id"},
                "metric_value_kwargs": {"bins": [397433.0, 4942918.5, 9488404.0]},
            },
            "num_batches": 1,
        },
    }

    parameter_node: ParameterNode = get_parameter_value_and_validate_return_type(
        domain=domain,
        parameter_reference=parameter_builder.json_serialized_fully_qualified_parameter_name,
        expected_return_type=None,
        variables=variables,
        parameters=parameters,
    )

    assert parameter_node == expected_parameter_node_as_dict


@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice_null_bins(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "my_column"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    with mock.patch(
        "great_expectations.expectations.metrics.column_aggregate_metrics.column_partition._get_column_partition_using_metrics",
        return_value=None,
    ):
        with pytest.raises(gx_exceptions.ProfilerExecutionError) as excinfo:
            variables: Optional[ParameterContainer] = None
            # noinspection PyUnusedLocal
            parameter_builder.build_parameters(
                domain=domain,
                variables=variables,
                parameters=parameters,
                batch_request=batch_request,
                runtime_configuration=None,
            )

        assert (
            "Partitioning values for HistogramSingleBatchParameterBuilder by column_partition_metric_single_batch_parameter_builder into bins encountered empty or non-existent elements."
            in str(excinfo.value)
        )


@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice_nan_valued_bins(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "my_column"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    with mock.patch(
        "great_expectations.execution_engine.pandas_batch_data.PandasBatchData.dataframe",
        new_callable=mock.PropertyMock,
        return_value=pd.DataFrame(
            {
                "my_column": [
                    None,
                    None,
                    None,
                    None,
                    None,
                ],
            }
        ),
    ):
        variables: Optional[ParameterContainer] = None
        parameter_builder.build_parameters(
            domain=domain,
            variables=variables,
            parameters=parameters,
            batch_request=batch_request,
            runtime_configuration=None,
        )

        expected_parameter_node_as_dict: dict = {
            "value": {"bins": [None], "weights": [], "tail_weights": [0.5, 0.5]},
            "details": {
                "metric_configuration": {
                    "metric_name": "column.histogram",
                    "domain_kwargs": {"column": "my_column"},
                    "metric_value_kwargs": {"bins": [None]},
                },
                "num_batches": 1,
            },
        }

        parameter_node: ParameterNode = get_parameter_value_and_validate_return_type(
            domain=domain,
            parameter_reference=parameter_builder.json_serialized_fully_qualified_parameter_name,
            expected_return_type=None,
            variables=variables,
            parameters=parameters,
        )

        assert parameter_node == expected_parameter_node_as_dict


@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice_wrong_type_bins(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "event_ts"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    variables: Optional[ParameterContainer] = None
    with pytest.raises(gx_exceptions.ProfilerExecutionError) as excinfo:
        # noinspection PyUnusedLocal
        parameter_builder.build_parameters(
            domain=domain,
            variables=variables,
            parameters=parameters,
            batch_request=batch_request,
            runtime_configuration=None,
        )

    assert (
        "Partitioning values for HistogramSingleBatchParameterBuilder by column_partition_metric_single_batch_parameter_builder did not yield bins of supported data type."
        in str(excinfo.value)
    )


@pytest.mark.parametrize(
    "column_values,bins,",
    [
        pytest.param(
            [
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
            ],
            [0.0],
        ),
        pytest.param(
            [
                1,
                1,
                1,
                1,
            ],
            [1.0],
        ),
    ],
)
@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice_reduced_bins_count(
    column_values,
    bins,
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    batch_request: dict = {
        "datasource_name": "alice_columnar_table_single_batch_datasource",
        "data_connector_name": "alice_columnar_table_single_batch_data_connector",
        "data_asset_name": "alice_columnar_table_single_batch_data_asset",
    }

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    metric_domain_kwargs: dict = {"column": "my_column"}
    domain = Domain(
        domain_type=MetricDomainTypes.COLUMN,
        domain_kwargs=metric_domain_kwargs,
        rule_name="my_rule",
    )

    parameter_container = ParameterContainer(parameter_nodes=None)
    parameters: Dict[str, ParameterContainer] = {
        domain.id: parameter_container,
    }

    assert parameter_container.parameter_nodes is None

    variables: Optional[ParameterContainer] = None

    expected_parameter_node_as_dict: dict
    parameter_node: ParameterNode

    with mock.patch(
        "great_expectations.execution_engine.pandas_batch_data.PandasBatchData.dataframe",
        new_callable=mock.PropertyMock,
        return_value=pd.DataFrame(
            {
                "my_column": column_values,
            }
        ),
    ):
        parameter_builder.build_parameters(
            domain=domain,
            variables=variables,
            parameters=parameters,
            batch_request=batch_request,
            runtime_configuration=None,
        )

        expected_parameter_node_as_dict = {
            "value": {"bins": bins, "weights": [], "tail_weights": [0.5, 0.5]},
            "details": {
                "metric_configuration": {
                    "metric_name": "column.histogram",
                    "domain_kwargs": {"column": "my_column"},
                    "metric_value_kwargs": {
                        "bins": bins,
                    },
                },
                "num_batches": 1,
            },
        }

        parameter_node = get_parameter_value_and_validate_return_type(
            domain=domain,
            parameter_reference=parameter_builder.json_serialized_fully_qualified_parameter_name,
            expected_return_type=None,
            variables=variables,
            parameters=parameters,
        )

        assert parameter_node == expected_parameter_node_as_dict


@pytest.mark.integration
def test_histogram_single_batch_parameter_builder_alice_check_serialized_keys(
    alice_columnar_table_single_batch_context,
):
    data_context: DataContext = alice_columnar_table_single_batch_context

    parameter_builder: ParameterBuilder = HistogramSingleBatchParameterBuilder(
        name="my_name",
        bins="auto",
        evaluation_parameter_builder_configs=None,
        data_context=data_context,
    )

    # Note: "evaluation_parameter_builder_configs" is not one of "ParameterBuilder" formal property attributes.
    assert set(parameter_builder.to_json_dict().keys()) == {
        "class_name",
        "module_name",
        "name",
        "evaluation_parameter_builder_configs",
    }
